<?php

namespace Drupal\entity_share_cron;

use Drupal\Core\Queue\QueueFactory;
use Drupal\Component\Serialization\Json;
use Drupal\entity_share_client\Entity\Remote;
use Drupal\entity_share_client\Service\RemoteManagerInterface;
use Drupal\entity_share_client\Service\JsonapiHelperInterface;

/**
 * Entity Share Cron service.
 */
class EntityShareCronService implements EntityShareCronServiceInterface {

  /**
   * Remote manager service.
   *
   * @var \Drupal\entity_share_client\Service\RemoteManagerInterface
   */
  protected $remoteManager;

  /**
   * JSON API helper service.
   *
   * @var \Drupal\entity_share_client\Service\JsonapiHelperInterface
   */
  protected $jsonapiHelper;

  /**
   * Queue service.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * Constructor.
   *
   * @param \Drupal\entity_share_client\Service\RemoteManagerInterface $remote_manager
   *   Remote manager to get channels infos from.
   * @param \Drupal\entity_share_client\Service\JsonapiHelperInterface $jsonapi_helper
   *   Helps to prepare JSON responses.
   * @param \Drupal\Core\Queue\QueueFactory $queue_factory
   *   Queue service.
   */
  public function __construct(RemoteManagerInterface $remote_manager,
                              JsonapiHelperInterface $jsonapi_helper,
                              QueueFactory $queue_factory) {
    $this->remoteManager = $remote_manager;
    $this->jsonapiHelper = $jsonapi_helper;
    $this->queueFactory = $queue_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function enqueue($remote_id, $channel_id, array $channel_info) {
    $queue_name = EntityShareCronServiceInterface::PENDING_QUEUE_NAME;
    $queue = $this->queueFactory->get($queue_name);
    $item = [
      'remote_id' => $remote_id,
      'channel_id' => $channel_id,
      'channel_info' => $channel_info,
    ];
    $queue->createItem($item);
  }

  /**
   * {@inheritdoc}
   */
  public function sync($remote_id, $channel_id, array $channel_info, $page_limit = 0) {
    // Collects entities to import from each page.
    $data_to_import = [];
    $next_page = 1;
    $url = $channel_info['url'];
    while ($url) {
      // Performs request to get the list of entities.
      $page_data = $this->getPage($remote_id, $channel_id, $url);
      $data_to_import = array_merge($data_to_import, $page_data['data']);
      $next_page++;

      if ($url = $page_data['next']) {
        if ($page_limit != 0 && $next_page > $page_limit) {
          // Enqueues the next page.
          $next_info = $channel_info;
          $next_info['url'] = $url;
          $this->enqueue($remote_id, $channel_id, $next_info);
          $url = FALSE;
        }
      }
    }

    // Imports the data.
    return $this->jsonapiHelper->importEntityListData($data_to_import);
  }

  /**
   * Returns the page data to import entities.
   *
   * @param string $remote_id
   *   The ID of the remote the channel belongs to.
   * @param string $channel_id
   *   The ID of the channel to be synchronized.
   * @param string $url
   *   The URL of the page.
   *
   * @return array
   *   An associative array with the following keys:
   *   - data => parsed JSON of entities to import.
   *   - next => URL of the next page.
   */
  protected function getPage($remote_id, $channel_id, $url) {
    $data = [
      'data' => [],
      'next' => FALSE,
    ];

    if ($remote = Remote::load($remote_id)) {
      // Makes request to get profiles.
      $this->jsonapiHelper->setRemote($remote);
      $http_client = $this->remoteManager->prepareJsonApiClient($remote);
      $json_response = $http_client->get($url)
                                   ->getBody()
                                   ->getContents();

      // Parses the JSON of entities to import.
      $json = Json::decode($json_response);
      $data['data'] = $this->jsonapiHelper->prepareData($json['data']);

      // Gets the URL of the next page.
      $data['next'] = !empty($json['links']['next']) ? $json['links']['next'] : FALSE;
    }

    return $data;
  }

}
