<?php

namespace Drupal\entity_share_cron\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\entity_share_client\Entity\Remote;
use Drupal\entity_share_client\Service\RemoteManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Module settings form.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * Remote manager service.
   *
   * @var \Drupal\entity_share_client\Service\RemoteManagerInterface
   */
  protected $remoteManager;

  /**
   * Constructs a \Drupal\system\ConfigFormBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\entity_share_client\Service\RemoteManagerInterface $remote_manager
   *   Remote manager service.
   */
  public function __construct(ConfigFactoryInterface $config_factory,
                              RemoteManagerInterface $remote_manager) {
    parent::__construct($config_factory);
    $this->remoteManager = $remote_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('config.factory'),
                      $container->get('entity_share_client.remote_manager'));
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'entity_share_cron_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function getEditableConfigNames() {
    return ['entity_share_cron.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('entity_share_cron.settings');
    $form['cron_interval'] = [
      '#type' => 'number',
      '#title' => $this->t('Execution interval'),
      '#description' => $this->t('Minimum interval between consecutive executions in seconds.'),
      '#min' => 60,
      '#step' => 60,
      '#default_value' => $config->get('cron_interval'),
    ];
    $form['remotes'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Enabled remotes and channels'),
      '#description' => $this->t('Only selected remotes and channels will be synchronized on Cron executions.'),
      '#tree' => TRUE,
    ];
    $remotes_config = $config->get('remotes');
    foreach (Remote::loadMultiple() as $remote_id => $remote) {
      // Adds a checkbox to enable/disable remote synchronization.
      $form['remotes'][$remote_id] = [
        '#type' => 'container',
      ];
      $remote_enabled_default = !empty($remotes_config[$remote_id]['enabled']);
      $form['remotes'][$remote_id]['enabled'] = [
        '#type' => 'checkbox',
        '#title' => $remote->label(),
        '#default_value' => $remote_enabled_default,
        '#ajax' => [
          'callback' => [$this, 'remoteCheckboxCallback'],
          'wrapper' => "channels-$remote_id",
        ],
      ];

      $form['remotes'][$remote_id]['channels'] = [
        '#type' => 'container',
        '#prefix' => "<div id=\"channels-$remote_id\" class=\"channels-container\">",
        '#suffix' => '</div>',
      ];
      $remote_enabled = $form_state->getValue(['remotes', $remote_id, 'enabled']);
      if (!isset($remote_enabled) && $remote_enabled_default || $remote_enabled) {
        $channels = $this->remoteManager->getChannelsInfos($remote);
        foreach ($channels as $channel_id => $channel_info) {
          $form['remotes'][$remote_id]['channels'][$channel_id] = [
            '#type' => 'container',
          ];
          $form['remotes'][$remote_id]['channels'][$channel_id]['enabled'] = [
            '#type' => 'checkbox',
            '#title' => $channel_info['label'],
            '#default_value' => !empty($remotes_config[$remote_id]['channels'][$channel_id]['enabled']),
          ];
          $form['remotes'][$remote_id]['channels'][$channel_id]['url'] = [
            '#type' => 'hidden',
            '#value' => $channel_info['url'],
          ];
          $form['remotes'][$remote_id]['channels'][$channel_id]['url_uuid'] = [
            '#type' => 'hidden',
            '#value' => $channel_info['url_uuid'],
          ];
        }
      }
    }
    $form['#attached']['library'][] = 'entity_share_cron/settings_form';
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Checks if every enabled remote has at least one channel enabled.
    $remotes = $form_state->getValue('remotes');
    foreach ($remotes as $remote_id => $remote_config) {
      if (!empty($remote_config['enabled'])) {
        // Searches for an enabled channel.
        $channel_enabled = FALSE;
        $channels = isset($remote_config['channels']) ? $remote_config['channels'] : [];
        foreach ($channels as $channel_id => $channel_config) {
          if (!empty($channel_config['enabled'])) {
            $channel_enabled = TRUE;
            break;
          }
        }

        // Shows an error if no enabled channel could be found.
        if (!$channel_enabled) {
          $element = &$form['remotes'][$remote_id];
          $form_state->setError($element, $this->t('No channels enabled for remote %remote.', [
            '%remote' => $element['enabled']['#title'],
          ]));
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('entity_share_cron.settings')
         ->set('cron_interval', $form_state->getValue('cron_interval'))
         ->set('remotes', $form_state->getValue('remotes'))
         ->save();
    parent::submitForm($form, $form_state);
  }

  /**
   * Ajax callback for the remotes' checkboxes.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function remoteCheckboxCallback(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $remote_id = $triggering_element['#parents'][1];
    return $form['remotes'][$remote_id]['channels'];
  }

}
