<?php

namespace Drupal\Tests\entity_share_cron\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\entity_share_client\Entity\Remote;
use Drupal\entity_share_cron\EntityShareCronService;
use Drupal\entity_share_cron\EntityShareCronServiceInterface;

/**
 * @coversDefaultClass \Drupal\entity_share_cron\EntityShareCronService
 * @group entity_share_cron
 */
class EntityShareCronServiceTest extends UnitTestCase {

  const REMOTE_ID = 'test_remote';
  const CHANNEL_ID = 'test_channel';
  const CHANNEL_URL = 'https://example.com';

  /**
   * The service under test.
   *
   * @var \Drupal\entity_share_cron\EntityShareCronServiceInterface
   */
  protected $service;

  /**
   * Mocked queue.
   *
   * @var \Drupal\Core\Queue\QueueInterface
   */
  protected $queue;

  /**
   * The number of requests made during the test to customize returned bodies.
   *
   * @var int
   */
  protected $requestCount;

  /**
   * The number of pages that the response should have.
   *
   * @var int
   */
  protected $responsePages;

  /**
   * Tests the sync() method running through all pages.
   */
  public function testSynchronizationAllPages() {
    $this->responsePages = 3;

    // No page can be enqueued.
    $this->queue->expects($this->never())
                ->method('createItem');

    $info = ['url' => self::CHANNEL_URL];
    $imported = $this->service->sync(self::REMOTE_ID, self::CHANNEL_ID, $info);

    $this->assertCount(3, $imported);
    $this->assertContains('request_0', $imported);
    $this->assertContains('request_1', $imported);
    $this->assertContains('request_2', $imported);
  }

  /**
   * Tests the sync() method with page limit.
   */
  public function testSynchronizationWithPageLimit() {
    $this->responsePages = 3;

    // Sets expectation to get third page enqueued.
    $expected_item = [
      'remote_id' => self::REMOTE_ID,
      'channel_id' => self::CHANNEL_ID,
      'channel_info' => [
        'url' => 'page_2',
      ],
    ];
    $this->queue->expects($this->once())
                ->method('createItem')
                ->with($this->equalTo($expected_item));

    $info = ['url' => self::CHANNEL_URL];
    $imported = $this->service->sync(self::REMOTE_ID, self::CHANNEL_ID, $info, 2);

    $this->assertCount(2, $imported);
    $this->assertContains('request_0', $imported);
    $this->assertContains('request_1', $imported);
  }

  /**
   * Returns the body of a request for a page.
   *
   * @return \Psr\Http\Message\StreamInterface
   *   The mocked body.
   */
  public function getResponseBody() {
    $this->responsePages -= 1;
    $json = [
      'data' => [
        'request_' . $this->requestCount++,
      ],
      'links' => [
        'next' => $this->responsePages > 0 ? 'page_' . $this->requestCount : FALSE,
      ],
    ];
    $contents = json_encode($json);
    $stream = $this->getMock('Psr\Http\Message\StreamInterface');
    $stream->expects($this->any())
           ->method('getContents')
           ->will($this->returnValue($contents));
    return $stream;
  }

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    $this->requestCount = 0;

    // Mocks an entity type repository and an entity type manager to load
    // remotes.
    $remote = new Remote([
      'id' => self::REMOTE_ID,
    ], 'remote');
    $remote_storage = $this->getMockBuilder('Drupal\Core\Config\Entity\ConfigEntityStorageInterface')
                           ->disableOriginalConstructor()
                           ->getMock();
    $remote_storage->expects($this->any())
                   ->method('load')
                   ->with($this->equalTo(self::REMOTE_ID))
                   ->will($this->returnValue($remote));
    $entity_type_manager = $this->getMock('Drupal\Core\Entity\EntityTypeManagerInterface');
    $entity_type_manager->expects($this->any())
                        ->method('getStorage')
                        ->with($this->equalTo('Remote'))
                        ->will($this->returnValue($remote_storage));
    $entity_type_repository = $this->getMock('Drupal\Core\Entity\EntityTypeRepositoryInterface');
    $entity_type_repository->expects($this->any())
                           ->method('getEntityTypeFromClass')
                           ->will($this->returnValue('Remote'));
    $container = new ContainerBuilder();
    $container->set('entity_type.manager', $entity_type_manager);
    $container->set('entity_type.repository', $entity_type_repository);
    \Drupal::setContainer($container);

    // Mocks a remote manager.
    $response = $this->getMock('Psr\Http\Message\ResponseInterface');
    $response->expects($this->any())
             ->method('getBody')
             ->will($this->returnCallback([$this, 'getResponseBody']));
    $http_client = $this->getMock('GuzzleHttp\Client');
    $http_client->expects($this->atLeastOnce())
                ->method('__call')
                ->with($this->equalTo('get'))
                ->will($this->returnValue($response));
    $remote_manager = $this->getMock('Drupal\entity_share_client\Service\RemoteManagerInterface');
    $remote_manager->expects($this->any())
                   ->method('prepareJsonApiClient')
                   ->will($this->returnValue($http_client));

    // Mocks a JSON API Helper.
    $jsonapi_helper = $this->getMock('Drupal\entity_share_client\Service\JsonapiHelperInterface');
    $jsonapi_helper->expects($this->any())
                   ->method('prepareData')
                   ->will($this->returnArgument(0));
    $jsonapi_helper->expects($this->once())
                   ->method('importEntityListData')
                   ->will($this->returnArgument(0));

    // Mocks a queue factory.
    $this->queue = $this->getMock('Drupal\Core\Queue\QueueInterface');
    $queue_factory = $this->getMockBuilder('Drupal\Core\Queue\QueueFactory')
                          ->disableOriginalConstructor()
                          ->setMethods(['get'])
                          ->getMock();
    $queue_factory->expects($this->any())
                  ->method('get')
                  ->with($this->equalTo(EntityShareCronServiceInterface::PENDING_QUEUE_NAME))
                  ->will($this->returnValue($this->queue));

    // Instantiates a service to test.
    $this->service = new EntityShareCronService($remote_manager, $jsonapi_helper, $queue_factory);
  }

}
